/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  ManagerPanel.java
 *  2003-2004 Vouriot Thierry
 *  yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package snmpmanager.gui;

import java.util.*;
import java.awt.event.*;
import java.awt.*;
import java.net.*;
import java.io.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.border.*;
import javax.swing.event.*;
import snmpmanager.*;
import snmpmanager.images.Icons;
import toolkit.*;
import toolkit.gui.*;
import snmp.*;

/** Panel permettant de surveiller et d'interroger plusieurs agents SNMP */
public class ManagerPanel extends DefaultPanel {
    
    /** Colonnes de la JTable */
    private static final int COL_STATUS = 0;
    private static final int COL_SYS = 1;
    private static final int COL_IP = 2;
    private static final int COL_COMMUNITY = 3;
    private static final int COL_NAME = 4;
    
    /** Systmes */
    private static final int SYS_WIN = 0;
    private static final int SYS_LINUX = 1;
    private static final int SYS_MAC = 2;
    private static final int SYS_UNKNOW = -1;
    
    /* Status */
    private static final int STATUS_CONNECT = 0;
    private static final int STATUS_DISCONNECT = -1;
    private static final int STATUS_UNKNOW = 2;
    
    private final JTextArea m_infosTxt = new JTextArea(7,1);
    private final RolloverButton m_delBtn = new RolloverButton("Delete",false);
    private final RolloverButton m_refreshBtn = new RolloverButton("Refresh",false);
    private final RolloverButton m_inquisitorBtn = new RolloverButton("Inquisitor",false);
    
    private final RolloverButton m_addBtn = new RolloverButton("Add",false);
    private final JTextField m_txtAddress = new JTextField("127.0.0.1");
    private final JTextField m_txtCommunity = new JTextField("public");
    private final JTextField m_txtName = new JTextField();
    
    private final JTable m_hostsTable = new JTable();
    protected final HostsVector m_hosts = new HostsVector();
    private final HostsTableModel m_tableModel = new HostsTableModel();
    
    private PingSNMPDaeamon m_daemon = new PingSNMPDaeamon(m_hosts, m_hostsTable);
    
    /** Infos  rcuperer pour les hosts */
    private String [][]m_infosOid = {   {"Description : ", "1.3.6.1.2.1.1.1.0"},
    {"Up time : ", "1.3.6.1.2.1.1.3.0"},
    {"Contact : ", "1.3.6.1.2.1.1.4.0"},
    {"Name : ", "1.3.6.1.2.1.1.5.0"},
    {"Location : ", "1.3.6.1.2.1.1.6.0"}
    };
    
    /** Constructeur
     * @param logPanel nouveau panel de log
     * @param frame fentre principale de l'application
     */
    ManagerPanel(MainFrame frame, LogPanel logPanel) {
        super(frame, logPanel);
        this.setLayout(new BorderLayout());
        
        JPanel panelSouth = new JPanel(new BorderLayout());
        JPanel panelCenter = new JPanel(new BorderLayout());
        JPanel panelEast = new JPanel(new BorderLayout());
        
        JScrollPane js = new JScrollPane(m_infosTxt);
        js.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder("Host informations"),
        BorderFactory.createEmptyBorder(0, 0, 0, 0)
        ));
        m_infosTxt.setEditable(false);
        m_infosTxt.setBackground(panelSouth.getBackground());
        m_infosTxt.setLineWrap(true);
        panelSouth.add(js, BorderLayout.CENTER);
        
        JPanel panelNew = new JPanel(new GridLayout(8,1));
        panelNew.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder("New host"),
        BorderFactory.createEmptyBorder(0, 0, 0, 0)
        ));
        m_txtAddress.setPreferredSize(new Dimension(150, 20));
        panelNew.add(new JLabel("IP Address :"));
        panelNew.add(m_txtAddress);
        panelNew.add(new JLabel("Community :"));
        panelNew.add(m_txtCommunity);
        panelNew.add(new JLabel("Name :"));
        panelNew.add(m_txtName);
        panelNew.add(new JPanel());
        panelNew.add(m_addBtn);
        
        panelEast.add(panelNew,BorderLayout.NORTH);
        panelEast.add(new JPanel(),BorderLayout.CENTER);
        
        panelCenter.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder("Hosts list"),
        BorderFactory.createEmptyBorder(0, 0, 0, 0)
        ));
        
        JPanel panelBtns = new JPanel(new FlowLayout(FlowLayout.RIGHT,2,1));
        panelBtns.add(m_delBtn);
        panelBtns.add(m_inquisitorBtn);
        panelBtns.add(m_refreshBtn);
        
        panelCenter.add(new JScrollPane(m_hostsTable), BorderLayout.CENTER);
        panelCenter.add(panelBtns, BorderLayout.SOUTH);
        
        this.add(panelSouth,BorderLayout.SOUTH);
        this.add(panelEast,BorderLayout.EAST);
        this.add(panelCenter,BorderLayout.CENTER);
        
        m_hosts.add("127.0.0.1","public","Localhost");
        m_hosts.load();
        refreshTable();
        m_hostsTable.setColumnSelectionAllowed(false);
        m_hostsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        m_hostsTable.setRowHeight(19);
        
        m_addBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                System.out.println("Action: add host");
                if (m_txtName.getText().trim().equals("")) m_logPanel.logMessage("Name not valid");
                else if (Utilities.isValidIPAddress(m_txtAddress.getText()) && !m_txtCommunity.getText().trim().equals("")) {
                    m_hosts.add(m_txtAddress.getText().trim(), m_txtCommunity.getText(), m_txtName.getText());
                    refreshTable();
                    m_daemon.interrupt();
                    if (!m_daemon.isAlive()) m_daemon.start();
                }
                else m_logPanel.logMessage("IP Address or community not valid");
            }
        });
        
        m_delBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                int i = m_hostsTable.getSelectedRow();
                if (i != -1) {
                    m_hosts.removeElementAt(i);
                    refreshTable();
                    m_daemon.interrupt();
                    if (!m_daemon.isAlive()) m_daemon.start();
                }
            }
        });
        
        m_refreshBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                m_daemon.interrupt();
                if (!m_daemon.isAlive()) m_daemon.start();
                m_hostsTable.getSelectionModel().clearSelection();
                m_infosTxt.setText("");
            }
        });
        
        m_inquisitorBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                int row = m_hostsTable.getSelectedRow();
                if (row != -1) {
                    String ip = (String)m_tableModel.getValueAt(row, COL_IP);
                    String com = (String)m_tableModel.getValueAt(row, COL_COMMUNITY);
                    m_mainFrame.getTabbedPane().m_inquisitorPanel.setFields(ip, com);
                    m_mainFrame.getTabbedPane().setSelectedIndex(1);
                }
            }
        });
        
        m_hostsTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                m_runThread = new Thread() {
                    public void run() {
                        int row = m_hostsTable.getSelectedRow();
                        if (row != -1)
                            try {
                                m_infosTxt.setText("");
                                
                                String community = (String)m_tableModel.getValueAt(row, COL_COMMUNITY);
                                int version = 0;	// SNMPv1
                                InetAddress hostAddress = InetAddress.getByName((String)m_tableModel.getValueAt(row, COL_IP));
                                SNMPv1CommunicationInterface comInterface = new SNMPv1CommunicationInterface(version, hostAddress, community);
                                comInterface.setSocketTimeout(3000);
                                
                                String itemID;
                                StringBuffer res = new StringBuffer();
                                for (int i=0; i<m_infosOid.length; i++) {
                                    itemID = m_infosOid[i][1];
                                    SNMPVarBindList newVars = comInterface.getMIBEntry(itemID);
                                    
                                    SNMPSequence pair = (SNMPSequence)(newVars.getSNMPObjectAt(0));
                                    SNMPObjectIdentifier snmpOID = (SNMPObjectIdentifier)pair.getSNMPObjectAt(0);
                                    SNMPObject snmpValue = pair.getSNMPObjectAt(1);
                                    
                                    String typeString = snmpValue.getClass().getName();
                                    
                                    if (typeString.equals("snmp.SNMPOctetString")) {
                                        String snmpString = snmpValue.toString();
                                        
                                        // truncate at first null character
                                        int nullLocation = snmpString.indexOf('\0');
                                        if (nullLocation >= 0)
                                            snmpString = snmpString.substring(0,nullLocation);
                                        
                                        res.append(m_infosOid[i][0] + snmpString + "\n");
                                    }
                                    else {
                                        res.append(m_infosOid[i][0] + snmpValue + "\n");
                                    }
                                }
                                m_infosTxt.setText(res.toString());
                            }
                            catch(InterruptedIOException e) { }
                        catch(Exception e) { }
                    }
                };
                if (m_runThread.isAlive())
                    m_runThread.interrupt();
                m_runThread.start();  
            }
        });
        
        m_daemon.start();
    }
    
    /** Remplie la JTable */
    private void refreshTable() {
        
        String []col = {"status", "sys", "IP address", "community", "name"};
        Object tab[][] = new Object[m_hosts.size()][5];
        
        for (int i=0; i<m_hosts.size(); i++) {
            tab[i][0] = new Integer(STATUS_UNKNOW);
            tab[i][1] = new Integer(SYS_UNKNOW);
            tab[i][2] = m_hosts.getIP(i);
            tab[i][3] = m_hosts.getCommunity(i);
            tab[i][4] = m_hosts.getName(i);  
        }
        m_tableModel.setDataVector(tab, col);
        m_hostsTable.setModel(m_tableModel);
        
        this.m_hostsTable.getColumn("status").setResizable(false);
        this.m_hostsTable.getColumn("status").setPreferredWidth(50);
        this.m_hostsTable.getColumn("status").setMaxWidth(50);
        
        this.m_hostsTable.getColumn("sys").setResizable(false);
        this.m_hostsTable.getColumn("sys").setPreferredWidth(32);
        this.m_hostsTable.getColumn("sys").setMaxWidth(32);
        
        // pour le status on affiche un icon
        DefaultTableCellRenderer statusRenderer = new DefaultTableCellRenderer() {
            public void setValue(Object value) {
                int c = ((Integer)value).intValue();
                if (c == STATUS_CONNECT) setIcon(Utilities.getIcon(Icons.ICON_PLUG, SNMPManager.class));
                else if (c == STATUS_DISCONNECT) setIcon(Utilities.getIcon(Icons.ICON_NOTPLUG, SNMPManager.class));
                else if (c == STATUS_UNKNOW) setIcon(Utilities.getIcon(Icons.ICON_ISPLUG, SNMPManager.class));
            }
        };
        statusRenderer.setAlignmentX(JLabel.CENTER_ALIGNMENT);
        statusRenderer.setAlignmentY(JLabel.CENTER_ALIGNMENT);
        
        // pour le systeme on affiche un icon
        DefaultTableCellRenderer sysRenderer = new DefaultTableCellRenderer() {
            public void setValue(Object value) {
                int c = ((Integer)value).intValue();
                if (c == SYS_WIN) setIcon(Utilities.getIcon(Icons.ICON_WIN, SNMPManager.class));
                else if (c == SYS_LINUX) setIcon(Utilities.getIcon(Icons.ICON_LINUX, SNMPManager.class));
                else if (c == SYS_MAC) setIcon(Utilities.getIcon(Icons.ICON_MAC, SNMPManager.class));
                else if (c == SYS_UNKNOW) setIcon(Utilities.getIcon(Icons.ICON_UNKNOW, SNMPManager.class));
            }
        };
        sysRenderer.setAlignmentX(JLabel.CENTER_ALIGNMENT);
        sysRenderer.setAlignmentY(JLabel.CENTER_ALIGNMENT);
        
        TableColumn statusColumn = m_hostsTable.getColumn("status");
        statusRenderer.setHorizontalAlignment(JLabel.CENTER);
        statusRenderer.setVerticalAlignment(JLabel.CENTER);
        statusColumn.setCellRenderer(statusRenderer);
        
        TableColumn sysColumn = m_hostsTable.getColumn("sys");
        sysRenderer.setHorizontalAlignment(JLabel.CENTER);
        sysRenderer.setVerticalAlignment(JLabel.CENTER);
        sysColumn.setCellRenderer(sysRenderer);
    }
    
    /** Model de JTable */
    private class HostsTableModel extends DefaultTableModel { 
        HostsTableModel() {
            super();
        }  
        public HostsTableModel(Object [][] data, Object[] columnNames) {
            super(data, columnNames);
        }
        public Class getColumnClass(int columnIndex) {
            return Object.class;
        }
        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return false;
        }
    }
    
    /** Daemon permettant d'interroger tous les agents SNMP de la JTable toutes les 60 secondes */
    private class PingSNMPDaeamon extends Thread {
        
        private HostsVector m_hosts;
        private JTable m_table;
        
        PingSNMPDaeamon(HostsVector hv, JTable tbl) {
            m_table = tbl;
            m_hosts = hv;
            this.setPriority(Thread.MIN_PRIORITY);
        }
        
        public void run() {
            
            while (true) {
                System.out.println("Action: refresh");
                for (int i=0; i<m_hosts.size(); i++) {
                    try {
                        String community = m_hosts.getCommunity(i);
                        int version = 0;	// SNMPv1
                        InetAddress hostAddress = InetAddress.getByName(m_hosts.getIP(i));
                        SNMPv1CommunicationInterface m_comInterface = new SNMPv1CommunicationInterface(version, hostAddress, community);
                        
                        String itemID = "1.3.6.1.2.1.1.1.0"; // description
                        m_comInterface.setSocketTimeout(1500);
                        SNMPVarBindList newVars = m_comInterface.getMIBEntry(itemID);
                        
                        SNMPSequence pair = (SNMPSequence)(newVars.getSNMPObjectAt(0));
                        SNMPObjectIdentifier snmpOID = (SNMPObjectIdentifier)pair.getSNMPObjectAt(0);
                        SNMPObject snmpValue = pair.getSNMPObjectAt(1);
                        
                        String snmpString = snmpValue.toString();
                        
                        // truncate at first null character
                        int nullLocation = snmpString.indexOf('\0');
                        if (nullLocation >= 0)
                            snmpString = snmpString.substring(0,nullLocation);
                        
                        snmpString = snmpString.toLowerCase();
                        if (snmpString.indexOf("linux") != -1)
                            m_table.getModel().setValueAt(new Integer(SYS_LINUX), i,  COL_SYS);
                        else if (snmpString.indexOf("windows") != -1)
                            m_table.getModel().setValueAt(new Integer(SYS_WIN), i,  COL_SYS);
                        else if (snmpString.indexOf("macintosh") != -1)
                            m_table.getModel().setValueAt(new Integer(SYS_MAC), i,  COL_SYS);
                        else m_table.getModel().setValueAt(new Integer(SYS_UNKNOW), i,  COL_SYS);
                        
                        m_table.getModel().setValueAt(new Integer(STATUS_CONNECT), i,  COL_STATUS);
                    }
                    catch(Exception e) {
                        m_table.getModel().setValueAt(new Integer(STATUS_DISCONNECT), i,  COL_STATUS);
                        m_table.getModel().setValueAt(new Integer(SYS_UNKNOW), i,  COL_SYS);
                    }
                }
                try {
                    this.sleep(60000);
                }
                catch(InterruptedException ie) {  }
            }
        }
    }
}

