/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  DefaultPanel.java
 *  2003-2004 Vouriot Thierry
 *  yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package snmpmanager.gui;

import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import snmpmanager.*;
import snmpmanager.images.Icons;
import toolkit.gui.*;

/** Panel par dfaut utilis dans le <CODE>ManagerTabbedPane</CODE> */
public class DefaultPanel extends JPanel {
    
    /** Rfrence vers le panel de log */    
    protected LogPanel m_logPanel = null;
    
    /** Fentre principale */    
    protected MainFrame m_mainFrame = null;
    
    /** Thread pour effectuer des traitements */    
    protected Thread m_runThread = null;
    
    /** Constructeur */    
    protected DefaultPanel() { }
    
    /** Constructeur 
     * @param logPanel nouveau panel de log
     * @param frame fentre principale de l'application
     */    
    protected DefaultPanel(MainFrame frame, LogPanel logPanel) { 
        m_logPanel = logPanel;
        m_mainFrame = frame;
    }
    
    /** Change le panel de log
     * @param logPanel nouveau panel de log
     */    
    public void setLogPanel(LogPanel logPanel) {
        m_logPanel = logPanel;
    }
    
    /** Change la fentre principale de l'application
     * @param frame fentre principale de l'application
     */    
    public void setMainFrame(MainFrame frame) {
        m_mainFrame = frame;
    }
    
    /** Test si le panel est en train d'effectuer un traitement
     * @return vrai si le panel effectue un traitement
     */    
    public boolean threadIsRunning() {
        if (m_runThread != null && m_runThread.isAlive())
            return true;
        return false;
        
    }
    
    /** Arrte le traitement en cours du panel */    
    public void stopThread() {
        if (m_runThread != null) {
            m_logPanel.logMessage("Arrt du traitement : thread " + m_runThread.getName());
            m_runThread.interrupt();
            //m_runThread.stop();
            m_runThread = null;
        }
    }
}