/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  HostsVector.java
 *  2003-2004 Vouriot Thierry
 *  yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package snmpmanager.gui;

import java.util.*;
import java.io.*;

/** Vector contenant les adresses ip et les communities des hosts */
public class HostsVector extends Vector {
    
    /** Une entre dans le vecteur contient une ip, une community et un nom */
    class Entry {
        String ip;
        String com;
        String name;
        Entry(String i, String c, String n) {
            ip = i;
            com = c;
            name = n;
        }
    }
    
    /** Fichier o sont enregistrs les hosts */
    public static final String HOSTS_FILE = System.getProperty("user.home")+File.separator+"snmpmanager.txt";
    
    /** Ajoute une nouvelle entre
     * @param ip adresse ip
     * @param community communaut
     * @param name nom de l'entre
     */
    public void add(String ip, String community, String name) {
        boolean add = true;
        for (int i=0; i<this.size(); i++)
            if (this.getIP(i).equals(ip)) { add = false; break; }
        if (add) this.add(new Entry(ip, community, name));
    }
    
    /** Retourne l'adresse ip d'une entre
     * @param i index de l'entre dans le vector
     * @return adresse ip
     */
    public String getIP(int i) {
        return ((Entry)this.get(i)).ip;
    }
    
    /** Retourne la communaut d'une entre
     * @param i index de l'entre dans le vector
     * @return cummunity
     */
    public String getCommunity(int i) {
        return ((Entry)this.get(i)).com;
    }
    
    /** Retourne le nom d'une entre
     * @param i index de l'entre dans le vector
     * @return nom
     */
    public String getName(int i) {
        return ((Entry)this.get(i)).name;
    }
    
    /** Charge le vector  partir du fichier <CODE>HOSTS_FILE</CODE> */
    public void load() {
        File f = new File(HOSTS_FILE);
        if (f.exists()) {
            try {
                RandomAccessFile accessFile = new RandomAccessFile(f, "r");
                String line;
                StringTokenizer token;
                while ((line = accessFile.readLine()) != null) {
                    token = new StringTokenizer(line, "|");
                    String [] host = new String[3];
                    int i = 0;
                    while (token.hasMoreTokens() && i<3) {
                        host[i] = token.nextToken();
                        
                        i++;
                    }
                    if (i==3) this.add(host[0], host[1], host[2]);
                }
                accessFile.close();
            }
            catch (Exception e) {
                System.out.println("HostsVector::load : "+e);
            }
        }
    }
    
    /** Enregistre le vector dans le fichier <CODE>HOSTS_FILE</CODE> */
    public void save() {
        File f = new File(HOSTS_FILE);     
        try {
            if (!f.exists()) f.createNewFile();
            RandomAccessFile accessFile = new RandomAccessFile(f, "rw");
            accessFile.setLength(0);
            
            for (int i=0; i<this.size(); i++)
                accessFile.writeBytes(this.getIP(i)+"|"+this.getCommunity(i)+"|"+this.getName(i)+"\r\n");
            
            accessFile.close();    
        }
        catch (Exception e) {
            System.out.println("HostsVector::save : "+e);
        }
    }
}