/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  InquisitorPanel.java
 *  2003-2004 Vouriot Thierry
 *  yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package snmpmanager.gui;

import java.util.*;
import java.awt.event.*;
import java.awt.*;
import java.net.*;
import java.io.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import snmpmanager.*;
import snmpmanager.images.Icons;
import toolkit.*;
import toolkit.gui.*;
import snmp.*;

/** Panel permettant d'interroger ou de changer des oids d'un agent SNMP */
public class InquisitorPanel extends DefaultPanel {
    
    private final JTextArea m_txtResponses = new JTextArea();
    
    private final RolloverButton m_oidBtn = new RolloverButton("Get OID Value",false);
    private final RolloverButton m_oidNextBtn = new RolloverButton("Get Next OID",false);
    private final RolloverButton m_oidAllBtn = new RolloverButton("Get All OID",false);
    private final RolloverButton m_oidSetBtn = new RolloverButton("Set OID Value",false);
    private final RolloverButton m_cleanBtn = new RolloverButton("Clean responses",false);
  
    private final JTextField m_txtAddress = new JTextField("127.0.0.1");
    private final JTextField m_txtCommunity = new JTextField("public");
    private final JTextField m_txtOid = new JTextField();
    private final JTextField m_txtSetValue = new JTextField();
    
    private final JComboBox m_valueTypeCmb = new JComboBox();
    
    private SNMPv1CommunicationInterface m_comInterface;
    
    /** Constructeur
     * @param logPanel nouveau panel de log
     * @param frame fentre principale de l'application
     */
    InquisitorPanel(MainFrame frame, LogPanel logPanel) {
        super(frame, logPanel);
        this.setLayout(new BorderLayout());
        
        JPanel panelWest = new JPanel(new BorderLayout());
        JPanel panelCenter = new JPanel(new BorderLayout());
        
        m_txtResponses.setEditable(false);
        final JScrollPane js = new JScrollPane(m_txtResponses);
        js.getViewport().addChangeListener(new ChangeListener() {
            private int lastHeight;
            public void stateChanged(ChangeEvent e) {
                JViewport vp = (JViewport)e.getSource();
                int h = vp.getViewSize().height;
                if (h != lastHeight) { // i.e. an addition not just a user scrolling
                    lastHeight = h;
                    int x = h - vp.getExtentSize().height;
                    vp.setViewPosition(new Point(0, x));
                }
            }
        });
        
        panelCenter.add(js,BorderLayout.CENTER);
        panelCenter.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder("Responses"),
        BorderFactory.createEmptyBorder(0, 0, 0, 0)
        ));
        
        m_valueTypeCmb.addItem("SNMPInteger");
        m_valueTypeCmb.addItem("SNMPCounter32");
        m_valueTypeCmb.addItem("SNMPCounter64");
        m_valueTypeCmb.addItem("SNMPGauge32");
        m_valueTypeCmb.addItem("SNMPOctetString");
        m_valueTypeCmb.addItem("SNMPIPAddress");
        m_valueTypeCmb.addItem("SNMPNSAPAddress");
        m_valueTypeCmb.addItem("SNMPObjectIdentifier");
        m_valueTypeCmb.addItem("SNMPTimeTicks");
        m_valueTypeCmb.addItem("SNMPUInteger32");
        
        JPanel panelInfos = new JPanel(new GridLayout(10,1));
        panelInfos.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder("Informations"),
        BorderFactory.createEmptyBorder(0, 0, 0, 0)
        ));
        m_txtAddress.setPreferredSize(new Dimension(150, 20));
        panelInfos.add(new JLabel("IP Address :"));
        panelInfos.add(m_txtAddress);
        panelInfos.add(new JLabel("Community :"));
        panelInfos.add(m_txtCommunity);
        panelInfos.add(new JLabel("OID :"));
        panelInfos.add(m_txtOid);
        panelInfos.add(new JLabel("Value (for set) :"));
        panelInfos.add(m_txtSetValue);
        panelInfos.add(new JLabel("Value type :"));
        panelInfos.add(m_valueTypeCmb);
        
        JPanel panelButtons = new JPanel(new GridLayout(5,1));
        panelButtons.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder("Commands"),
        BorderFactory.createEmptyBorder(0, 0, 0, 0)
        ));
        panelButtons.add(m_oidBtn);
        panelButtons.add(m_oidNextBtn);
        panelButtons.add(m_oidAllBtn);
        panelButtons.add(m_oidSetBtn);
        panelButtons.add(m_cleanBtn);
        
        JPanel panelLeft = new JPanel(new BorderLayout());
        panelLeft.add(panelInfos,BorderLayout.NORTH);
        panelLeft.add(panelButtons,BorderLayout.SOUTH);
        
        panelWest.add(panelLeft,BorderLayout.NORTH);
        panelWest.add(new JPanel(),BorderLayout.CENTER);
        
        JScrollPane js2 = new JScrollPane(panelWest);
        js2.setBorder(new EmptyBorder(0, 0, 0, 0));
        JSplitPane split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,js2, panelCenter);
        split.setBorder(new EmptyBorder(0, 0, 0, 0));
        split.setDividerSize(0);
        
        this.add(split,BorderLayout.CENTER);
        
        m_cleanBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                System.out.println("Action: clean");
                cleanAll();
            }
        });
        
        m_oidAllBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                System.out.println("Action: getalloid");
                if (m_runThread != null && m_runThread.isAlive()) m_runThread.interrupt();
                m_runThread = new Thread("<getalloid>") {
                    public void run() {
                        try {
                            String community = m_txtCommunity.getText();
                            int version = 0;	// SNMPv1
                            InetAddress hostAddress = InetAddress.getByName(m_txtAddress.getText().trim());
                            m_comInterface = new SNMPv1CommunicationInterface(version, hostAddress, community);
                            m_comInterface.setSocketTimeout(6000);
                 
                            String itemID = "";
                            String retrievedID = "1.3.6.1.2.1";		// start point
                            
                            while (!Thread.interrupted() && !retrievedID.equals(itemID)) {
                                itemID = retrievedID;
                                
                                SNMPVarBindList newVars = m_comInterface.getNextMIBEntry(itemID);
                                
                                SNMPSequence pair = (SNMPSequence)(newVars.getSNMPObjectAt(0));
                                SNMPObjectIdentifier snmpOID = (SNMPObjectIdentifier)pair.getSNMPObjectAt(0);
                                SNMPObject snmpValue = pair.getSNMPObjectAt(1);
                                retrievedID = snmpOID.toString();
                                String typeString = snmpValue.getClass().getName();
                                
                                if (typeString.equals("snmp.SNMPOctetString")) {
                                    String snmpString = snmpValue.toString();
                                    
                                    // truncate at first null character
                                    int nullLocation = snmpString.indexOf('\0');
                                    if (nullLocation >= 0)
                                        snmpString = snmpString.substring(0,nullLocation);
                                    
                                    m_txtResponses.append("OID: " + snmpOID + "  Type: " + typeString + "\nValue: " + snmpString + "\n");
                                }
                                else {
                                    m_txtResponses.append("OID: " + snmpOID + "  Type: " + typeString + "\nValue: " + snmpValue);
                                    m_txtResponses.append("\n");
                                }
                            }
                        }
                        catch(InterruptedIOException e) {
                            m_logPanel.logMessage("Interrupted during retrieval: "+e.getMessage());
                        }
                        catch(Exception e) {
                            m_logPanel.logMessage("Exception during retrieval: "+e.getMessage());
                        }
                    }
                };
                m_runThread.setPriority(Thread.MIN_PRIORITY);
                if (Utilities.isValidIPAddress(m_txtAddress.getText()))
                    m_runThread.start();
                else m_logPanel.logMessage("IP Adress not valid");
            }
        });
        
        m_oidNextBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                System.out.println("Action: getnextoid");
                if (testInfos())
                try {
                    String community = m_txtCommunity.getText();
                    int version = 0;	// SNMPv1
                    InetAddress hostAddress = InetAddress.getByName(m_txtAddress.getText().trim());
                    m_comInterface = new SNMPv1CommunicationInterface(version, hostAddress, community);
                    m_comInterface.setSocketTimeout(6000);
                    StringTokenizer st = new StringTokenizer(m_txtOid.getText(), " ,;");
                    
                    while (st.hasMoreTokens()) {
                        String itemID = st.nextToken();
                        SNMPVarBindList newVars = m_comInterface.getNextMIBEntry(itemID);
                        SNMPSequence pair = (SNMPSequence)(newVars.getSNMPObjectAt(0));
                        SNMPObjectIdentifier snmpOID = (SNMPObjectIdentifier)pair.getSNMPObjectAt(0);
                        SNMPObject snmpValue = pair.getSNMPObjectAt(1);
                        String typeString = snmpValue.getClass().getName();
                        m_txtOid.setText(snmpOID.toString());
                        
                        if (typeString.equals("snmp.SNMPOctetString")) {
                            String snmpString = snmpValue.toString();
                            
                            // truncate at first null character
                            int nullLocation = snmpString.indexOf('\0');
                            if (nullLocation >= 0)
                                snmpString = snmpString.substring(0,nullLocation);
                            
                            m_txtResponses.append("OID: " + snmpOID + "  Type: " + typeString + "\nValue: " + snmpString + "\n");
                        }
                        else {
                            m_txtResponses.append("OID: " + snmpOID + "  Type: " + typeString + "\nValue: " + snmpValue);
                            m_txtResponses.append("\n");
                        }
                    }
                }
                catch(InterruptedIOException e) {
                    m_logPanel.logMessage("Interrupted during retrieval: "+e.getMessage());
                }
                catch(Exception e) {
                    m_logPanel.logMessage("Exception during retrieval: "+e.getMessage());
                }
            }
        });
        
        m_oidBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                System.out.println("Action: getoidvalue");
                if (testInfos())
                try {
                    String community = m_txtCommunity.getText();
                    int version = 0;	// SNMPv1
                    InetAddress hostAddress = InetAddress.getByName(m_txtAddress.getText().trim());
                    m_comInterface = new SNMPv1CommunicationInterface(version, hostAddress, community);
                    m_comInterface.setSocketTimeout(6000);
                    StringTokenizer st = new StringTokenizer(m_txtOid.getText(), " ,;");
                    
                    while (st.hasMoreTokens()) {
                        String itemID = st.nextToken();
                        SNMPVarBindList newVars = m_comInterface.getMIBEntry(itemID);
                        SNMPSequence pair = (SNMPSequence)(newVars.getSNMPObjectAt(0));
                        SNMPObjectIdentifier snmpOID = (SNMPObjectIdentifier)pair.getSNMPObjectAt(0);
                        SNMPObject snmpValue = pair.getSNMPObjectAt(1);
                        String typeString = snmpValue.getClass().getName();
                        
                        if (typeString.equals("snmp.SNMPOctetString")) {
                            String snmpString = snmpValue.toString();
                            
                            // truncate at first null character
                            int nullLocation = snmpString.indexOf('\0');
                            if (nullLocation >= 0)
                                snmpString = snmpString.substring(0,nullLocation);
                            
                            m_txtResponses.append("OID: " + snmpOID + "  Type: " + typeString + "\nValue: " + snmpString + "\n");
                        }
                        else {
                            m_txtResponses.append("OID: " + snmpOID + "  Type: " + typeString + "\nValue: " + snmpValue);
                            m_txtResponses.append("\n");
                        }
                    }
                }
                catch(InterruptedIOException e) {
                    m_logPanel.logMessage("Interrupted during retrieval: "+e.getMessage());
                }
                catch(Exception e) {
                    m_logPanel.logMessage("Exception during retrieval: "+e.getMessage());
                }
            }
        });
        
        m_oidSetBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                System.out.println("Action: setoidvalue");
                if (testInfos()) {
                    try {
                        String community = m_txtCommunity.getText();
                        int version = 0;	// SNMPv1
                        InetAddress hostAddress = InetAddress.getByName(m_txtAddress.getText().trim());
                        m_comInterface = new SNMPv1CommunicationInterface(version, hostAddress, community);
                        m_comInterface.setSocketTimeout(6000);
                        
                        String itemID = m_txtOid.getText();
                        String valueString = m_txtSetValue.getText();
                        String valueTypeString = (String)m_valueTypeCmb.getSelectedItem();
                        valueTypeString = "snmp." + valueTypeString;
                        
                        SNMPObject itemValue;
                        Class valueClass = Class.forName(valueTypeString);
                        itemValue = (SNMPObject)valueClass.newInstance();
                        itemValue.setValue(valueString);
                        
                        SNMPVarBindList newVars = m_comInterface.setMIBEntry(itemID, itemValue);  
                        SNMPSequence pair = (SNMPSequence)(newVars.getSNMPObjectAt(0));
                        SNMPObjectIdentifier snmpOID = (SNMPObjectIdentifier)pair.getSNMPObjectAt(0);
                        SNMPObject snmpValue = pair.getSNMPObjectAt(1);
                        String typeString = snmpValue.getClass().getName();
                        
                        m_txtResponses.append("OID: " + snmpOID + "  Type: " + typeString + "\nValue: " + snmpValue + "\n");
                    }
                    catch(InterruptedIOException e) {
                        m_logPanel.logMessage("Interrupted during retrieval: "+e.getMessage());
                    }
                    catch(Exception e) {
                        m_logPanel.logMessage("Exception during retrieval: "+e.getMessage());
                    }
                }
            }  
        });
    }
    
    /** Teste si les champs sont correctement remplis */
    private boolean testInfos() {
        boolean res=true;
        if (this.m_txtAddress.getText().trim().equals("")) { m_logPanel.logMessage("IP Adress not valid"); res = false; }
        if (!Utilities.isValidIPAddress(m_txtAddress.getText().trim())) { m_logPanel.logMessage("IP Adress not valid"); res = false; }
        if (this.m_txtCommunity.getText().trim().equals("")) { m_logPanel.logMessage("Community not valid"); res = false; }
        if (this.m_txtOid.getText().trim().equals("")) { m_logPanel.logMessage("OID not valid"); res = false; }
        return res;   
    }
    
    /** Rempli les champs ip et community
     * @param ip adresse ip
     * @param community communaut
     */
    public void setFields(String ip, String community) {
        this.m_txtAddress.setText(ip);
        this.m_txtCommunity.setText(community);    
    }
    
    /** Efface et rinitialise tous les composants du panel */
    public void cleanAll() {
        m_txtResponses.setText("");
    }
}