/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  MainFrame.java
 *  2003-2004 Vouriot Thierry
 *  yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package snmpmanager.gui;

import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.Date;
import java.text.SimpleDateFormat;
import javax.swing.*;
import toolkit.*;
import toolkit.gui.*;
import snmpmanager.*;
import snmpmanager.images.*;

/** Fentre principale de SNMPManager
 */
public class MainFrame extends JFrame {
    
    private LogPanel m_logPanel;
    private ManagerTabbedPane m_tabbedPane;
    
    /** Constructeur
     */
    public MainFrame() {
        SNMPManager.initUI();
        Utilities.changePlaf(this,"metal");
        
        this.setTitle("SNMPManager");
        this.setIconImage(this.getToolkit().createImage(Utilities.getURL(Icons.ICON_SNMPMANAGER, SNMPManager.class)));
        this.setSize(640, 600);
        this.setLocation(Utilities.getCenterComponent(this));
        
        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
                m_tabbedPane.m_managerPanel.m_hosts.save();
                System.exit(0);
            }
        });
        
        m_logPanel = new LogPanel(false);
        m_logPanel.logMessage("SNMPManager v" + SNMPManager.RELEASE + " (build " + SNMPManager.BUILD + ")");
        m_logPanel.logMessage(System.getProperty("os.name") + " " + System.getProperty("os.version") + " " + System.getProperty("os.arch")+" Java " + System.getProperty("java.version") + " " + System.getProperty("java.vendor"));
        m_logPanel.logMessage("Directory : " + SNMPManager.getAppliHome());
        String date = (new SimpleDateFormat("EEEE, d MMMM yyyy")).format(new Date());
        m_logPanel.logMessage("Started on " + date);
        m_logPanel.setPreferredSize(new Dimension(100, 78));
    
        m_tabbedPane = new ManagerTabbedPane(m_logPanel, this);
        
        Container contentPane = this.getContentPane();
        contentPane.setLayout(new BorderLayout());
        contentPane.add(m_tabbedPane,BorderLayout.CENTER);
        contentPane.add(m_logPanel,BorderLayout.SOUTH); 
    }
    
    /** Retourne le panel de log
     * @return panel de log
     */    
    public LogPanel getLogPanel() {
        return m_logPanel;
    }
    
    /** Retourne le <CODE>JTabbedPane</CODE>
     * @return tabbedPane
     */    
    public ManagerTabbedPane getTabbedPane() {
        return m_tabbedPane;
    }
}
