/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  TrapsPanel.java
 *  2003-2004 Vouriot Thierry
 *  yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package snmpmanager.gui;

import java.util.*;
import java.awt.event.*;
import java.awt.*;
import java.net.*;
import java.io.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import snmpmanager.*;
import snmpmanager.images.Icons;
import snmpmanager.sounds.Sounds;
import toolkit.*;
import toolkit.gui.*;
import snmp.*;

/** Panel permettant de capturer les traps SNMP envoyes par les agents */
public class TrapsPanel extends DefaultPanel implements SNMPTrapListener {
    
    private final JTextArea m_txtTraps = new JTextArea();
    private final RolloverButton m_cleanBtn = new RolloverButton("Clean",false);
    private int m_nbTraps = 0;
    
    private SNMPv1TrapListenerInterface m_trapListenerInterface;
    
    /** Constructeur
     * @param logPanel nouveau panel de log
     * @param frame fentre principale de l'application
     */
    TrapsPanel(MainFrame frame, LogPanel logPanel) {
        super(frame,  logPanel);
        this.setLayout(new BorderLayout());
        
        JPanel panelSouth = new JPanel(new BorderLayout());
        JPanel panelCenter = new JPanel(new BorderLayout());
        
        m_txtTraps.setEditable(false);
        final JScrollPane js = new JScrollPane(m_txtTraps);
        js.getViewport().addChangeListener(new ChangeListener() {
            private int lastHeight;
            public void stateChanged(ChangeEvent e) {
                JViewport vp = (JViewport)e.getSource();
                int h = vp.getViewSize().height;
                if (h != lastHeight) { // i.e. an addition not just a user scrolling
                    lastHeight = h;
                    int x = h - vp.getExtentSize().height;
                    vp.setViewPosition(new Point(0, x));
                }
            }
        });
        
        panelCenter.add(js,BorderLayout.CENTER);
        panelCenter.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder("Traps receive"),
        BorderFactory.createEmptyBorder(0, 0, 0, 0)
        ));
        
        panelSouth.add(m_cleanBtn, BorderLayout.EAST);
        
        this.add(panelSouth,BorderLayout.SOUTH);
        this.add(panelCenter,BorderLayout.CENTER);
        
        m_cleanBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                System.out.println("Action: clean");
                cleanAll();
            }
        });
        
        try {
            m_trapListenerInterface = new SNMPv1TrapListenerInterface();
            m_trapListenerInterface.addTrapListener(this);
            m_trapListenerInterface.startReceiving();
        }
        catch(Exception e) {
            m_logPanel.logMessage("Exception trapListener : "+e.getMessage());
        }
    }
    
    /** Efface et rinitialise tous les composants du panel */
    public void cleanAll() {
        m_txtTraps.setText("");
        m_nbTraps = 0;
        m_mainFrame.getTabbedPane().setTitleAt(2, "Traps("+(m_nbTraps)+")");
    }
    
    /** Traitement effectu lorsque l'on reoit une trap
     * @param trapPDU trap snmp
     */
    public void processTrap(SNMPTrapPDU trapPDU) {
        AudioFilePlayer player = new AudioFilePlayer(Utilities.getURL(Sounds.WAV_ALARM, SNMPManager.class));
        player.play();
        int generic = trapPDU.getGenericTrap();
        String address = trapPDU.getAgentAddress().toString();
        m_txtTraps.append("Got trap at " + Utilities.getTimestamp() +"\n");
        m_txtTraps.append("  enterprise OID:     " + trapPDU.getEnterpriseOID().toString() + "\n");
        m_txtTraps.append("  agent address:      " + address + "\n");
        m_txtTraps.append("  generic trap:       " + generic + " => " + getTrapType(generic) +"\n");
        m_txtTraps.append("  specific trap:      " + trapPDU.getSpecificTrap() + "\n");
        m_txtTraps.append("  timestamp:          " + trapPDU.getTimestamp() + "\n");
        m_txtTraps.append("  supplementary vars: " + trapPDU.getVarBindList().toString() + "\n");
        m_txtTraps.append("\n");
        m_logPanel.logMessage("Trap (" +getTrapType(generic)+ ") received from "+address);
        m_mainFrame.getTabbedPane().setTitleAt(2, "Traps("+(++m_nbTraps)+")");
    }
    
    /** Retourne la description d'un trap en fonction de son numro "generic"
     * @param genericTrap numro "generic"
     * @return description de la trap
     */
    public static String getTrapType(int genericTrap) {
        String res = "";
        switch (genericTrap) {    
            case 0: res = "Cold start"; break;
            case 1: res = "Warm start"; break;
            case 2: res = "Link down"; break;
            case 3: res = "Link up"; break;
            case 4: res = "Authentification failure"; break;
            case 5: res = "EGP neighbor loss"; break;
            case 6: res = "Enterprise specific"; break;
        }
        return res;
    }   
}