/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  LogTextArea.java
 *  2003-2004 Vouriot Thierry
 *  yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package toolkit.gui;

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;

/** Zone de text permettant de logger des infos */
public class LogTextArea extends JPanel implements Logger {
    
    boolean m_first = true;
    JTextArea m_text = new JTextArea();
    
    /** Constructeur */    
    public LogTextArea() {
        
        m_text.setEditable(false);
        m_text.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 5));
        
        JPanel p1 = new JPanel();
        p1.setBorder(BorderFactory.createTitledBorder("Log"));
        p1.setLayout(new BorderLayout());
        final JScrollPane js = new JScrollPane(m_text);
        p1.add(js, BorderLayout.CENTER);
        
        setLayout(new BorderLayout());
        add(p1, BorderLayout.CENTER);
        
        js.getViewport().addChangeListener(new ChangeListener() {
            private int lastHeight;
            public void stateChanged(ChangeEvent e) {
                JViewport vp = (JViewport)e.getSource();
                int h = vp.getViewSize().height;
                if (h != lastHeight) { // i.e. an addition not just a user scrolling
                    lastHeight = h;
                    int x = h - vp.getExtentSize().height;
                    vp.setViewPosition(new Point(0, x));
                }
            }
        });
    }
    
    /** Efface le log */    
    public void clean() {
        m_text.setText("");
        m_first = true;
    }
    
    /**
     * Sends the supplied message to the log area. The current timestamp will
     * be prepended.
     *
     * @param message a value of type 'String'
     */
    public void logMessage(String message) {
        
        if (m_first) {
            m_first = false;
        } else {
            m_text.append("\n");
        }
        m_text.append(message);
    }
    
    /** Non implment */    
    public void statusMessage(String message) {
    }
}

